import React from "react";
import { Route, Switch } from "react-router-dom";
import { useDispatch, useSelector } from "react-redux";
import { Button } from "antd";
import axios from "axios";
// Views
import Home from "./views/Home/Home";
import Dashboard from "./views/ProfileDashboard/Dashboard/Dashboard";
import Signin from "./views/Signin/Signin";
// Import action
import { authCheckState } from "./store/actions/authActions.js";
import RedirectProfileHome from "./components/Redirects/RedirectProfileHome";
// Import custom route
import ProfileDashboardRoute from "./routes/ProfileDashboard/ProfileDashboardRoute";
import OrganizationDashboardRoute from "./routes/OrganizationDashboard/OrganizationDashboardRoute";
// Another thingy just in case
import Projects from "./views/ProfileDashboard/Projects/Projects";
import ProjectOverview from "./views/ProfileDashboard/ProjectOverview/ProjectOverview";
import Servers from "./views/ProfileDashboard/Servers/Servers";
import ServerOverview from "./views/ProfileDashboard/ServerOverview/ServerOverview";
import NewServer from "./views/ProfileDashboard/NewServer/NewServer";
import NewProject from "./views/ProfileDashboard/NewProject/NewProject";
// Custom alert
import { createAlert } from "./store/actions/generalActions";
import Verification from "./views/Verification/Verification";
import SearchResult from "./views/SearchResult/SearchResult";
import ProfilePage from "./views/ProfilePage/ProfilePage";
import ProfileProjectOverview from "./views/ProfileProjectOverview/ProfileProjectOverview";
import Settings from "./views/ProfileDashboard/Settings/Settings";
import OrganizationDashboard from "./views/OrganizationDashboard/Dashboard/OrganizationDashboard";
import Organizations from "./views/ProfileDashboard/Organizations/Organizations";
import Members from "./views/OrganizationDashboard/Members/Members";
import Applicants from "./views/OrganizationDashboard/Applicants/Applicants";
import ProjectRequests from "./views/OrganizationDashboard/ProjectRequests/ProjectRequests";
import ProjectSettings from "./views/ProfileDashboard/ProjectSettings/ProjectSettings";
import SubmitProject from "./views/OrganizationDashboard/SubmitProject/SubmitProject";
import ProfileOrganizationOverview from "./views/ProfileOrganizationOverview/ProfileOrganizationOverview";
import ProfileOrganizationForm from "./views/ProfileOrganizationForm/ProfileOrganizationForm";
import Submissions from "./views/OrganizationDashboard/Submissions/Submissions";
import ModuleSelection from "./views/ProfileDashboard/AddModule/ModuleSelection";
import ServerSettings from "./views/ProfileDashboard/ServerSettings/ServerSettings";

/**
 * Handles the routing mechanism in the application
 */
function App() {
	// State
	const [doneAuthenticating, setDoneAuthenticating] = React.useState(false);
	// See if token is set in store
	const isAuthenticated = useSelector((state) => state.auth.token !== null);
	const isVerified = useSelector((state) => state.auth.isVerified);
	const isEnabled = useSelector((state) => state.general.isEnabled);

	const dispatch = useDispatch();

	React.useEffect(() => {
		// Used to update token state in redux store when page is "refreshed"
		dispatch(authCheckState());
		setDoneAuthenticating(true);
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, []);

	const text = (
		<div>
			To be able to create, deploy and share projects, you need to verify your account. Didn't
			get the email?{" "}
			<Button
				style={{ padding: 0 }}
				type="link"
				onClick={() => {
					const url =
						process.env.REACT_APP_APPLICATION_URL +
						process.env.REACT_APP_AUTHENTICATION_SERVER +
						"/auth/resendVerificationToken";

					axios
						.get(url)
						.then(() => {
							dispatch(
								createAlert(
									"New verification token sent",
									"Successfully sent a new verification token. Please check your inbox for a new verification email.",
									"success",
									true
								)
							);
						})
						.catch(() => {
							dispatch(
								createAlert(
									"Request failed",
									"Could not request a new verification token. Try again later or contact support here: contact@autopacker.no",
									"error",
									true
								)
							);
						});
				}}
			>
				Click here to resend
			</Button>
		</div>
	);

	React.useEffect(() => {
		if (isVerified === false) {
			dispatch(createAlert("Please verify your email address", text, "warning", true));
		}
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, [doneAuthenticating]);

	let router = null;
	if (isAuthenticated) {
		router = (
			<React.Fragment>
				<OrganizationDashboardRoute
					path="/organization/submissions/:organization"
					component={Submissions}
				/>
				<OrganizationDashboardRoute
					path="/organization/project-requests/:organization"
					component={ProjectRequests}
				/>
				<OrganizationDashboardRoute
					path="/organization/submit-project/:organization"
					component={SubmitProject}
				/>
				<OrganizationDashboardRoute
					path="/organization/dashboard/:organization"
					component={OrganizationDashboard}
				/>
				<OrganizationDashboardRoute
					path="/organization/applicants/:organization"
					component={Applicants}
				/>
				<OrganizationDashboardRoute
					path="/organization/members/:organization"
					component={Members}
				/>
				<ProfileDashboardRoute
					path="/profile/server-overview/:server/settings"
					component={ServerSettings}
				/>
				<ProfileDashboardRoute
					path="/profile/project-overview/:project/settings"
					component={ProjectSettings}
				/>
				<ProfileDashboardRoute
					exact
					path="/profile/project-overview/:project"
					component={ProjectOverview}
				/>
				<ProfileDashboardRoute
					exact
					path="/profile/server-overview/:server"
					component={ServerOverview}
				/>
				<ProfileDashboardRoute path="/profile/organizations" component={Organizations} />
				<ProfileDashboardRoute path="/profile/add-module" component={ModuleSelection} />
				<ProfileDashboardRoute path="/profile/add-server" component={NewServer} />
				<ProfileDashboardRoute path="/profile/dashboard" component={Dashboard} />
				<ProfileDashboardRoute path="/profile/projects" component={Projects} />
				<ProfileDashboardRoute path="/profile/settings" component={Settings} />
				<ProfileDashboardRoute path="/profile/servers" component={Servers} />
				<ProfileDashboardRoute path="/profile/new-project" component={NewProject} />
				<Route
					path="/organization/:organization/membership"
					component={ProfileOrganizationForm}
				/>
				<Route
					exact
					path="/organization/:organization"
					component={ProfileOrganizationOverview}
				/>
				<Route
					path="/account/:username/project/:project"
					component={ProfileProjectOverview}
				/>
				<Route path="/registrationConfirmation" component={Verification} />
				<Route exact path="/account/:username" component={ProfilePage} />
				<Route path="/signin" component={RedirectProfileHome} />
				<Route path="/search" component={SearchResult} />
				{/* This route is on the bottom of the list */}
				<Route exact path="/" component={Home} />
			</React.Fragment>
		);
	} else {
		router = (
			<React.Fragment>
				<Route
					exact
					path="/organization/:organization"
					component={ProfileOrganizationOverview}
				/>
				<Route
					path="/account/:username/project/:project"
					component={ProfileProjectOverview}
				/>
				<Route path="/registrationConfirmation" component={Verification} />
				<Route exact path="/account/:username" component={ProfilePage} />
				<Route path="/search" component={SearchResult} />
				<Route path="/signin" component={Signin} />
				{/* This route is on the bottom of the list */}
				<Route exact path="/" component={Home} />
			</React.Fragment>
		);
	}

	return (
		<div>
			<Switch>{router}</Switch>
		</div>
	);
}

export default App;
