import { Button, Form, Icon, Input, Layout, Radio, Tag, Tooltip } from "antd";
import { TweenOneGroup } from "rc-tween-one";
import React, { useEffect } from "react";
import { useDispatch, useSelector } from "react-redux";
import { createAlert } from "../../../../../store/actions/generalActions";
import { axiosRequest } from "../../../../../util/axiosRequest";

function RequestEditForm(props) {
	// Form state
	const [projectName, setProjectName] = React.useState("");
	const [desc, setDesc] = React.useState("");
	const [type, setType] = React.useState("");
	const [authors, setAuthors] = React.useState([]);
	const [links, setLinks] = React.useState([]);
	const [tags, setTags] = React.useState([]);
	const [comment, setComment] = React.useState("");

	// Controller state
	const [tagInput, setTagInput] = React.useState("");

	// Import sub components from antd
	const { TextArea } = Input;
	const { Content } = Layout;

	const { getFieldDecorator } = props.form;
	const { request, setOpenEditModal, toggleRefresh } = props;

	const organizationName = useSelector((state) => state.org.organizationName);
	const dispatch = useDispatch();

	useEffect(() => {
		setProjectName(request.project.name);
		setDesc(request.project.description);
		setType(request.project.type);
		setAuthors(request.project.authors.split(", "));
		setLinks(request.project.links.split(", "));
		setTags(request.project.tags.split(","));
		setComment(request.comment);
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, []);

	const removeTag = (removedTag) => {
		const tagsa = tags.filter((tag) => tag !== removedTag);
		setTags(tagsa);
	};

	const handleTagInputConfirm = () => {
		let tagsa = tags;
		if (tagInput && tagsa.indexOf(tagInput) === -1) {
			tagsa = [...tags, tagInput];
		}
		setTags(tagsa);
		setTagInput("");
	};

	const handleSubmit = (event) => {
		event.preventDefault();

		const url =
			process.env.REACT_APP_APPLICATION_URL +
			process.env.REACT_APP_GENERAL_API +
			"/organization/updateProjectSubmission";

		axiosRequest(
			"post",
			url,
			{},
			{
				organizationName: organizationName,
				projectName: projectName,
				desc: desc,
				type: type,
				authors: authors,
				links: links,
				tags: tags,
				comment: comment,
				projectId: request.project.id,
			},
			(response) => {
				dispatch(
					createAlert(
						"Project request updated",
						"You have successfully updated the project information. You should notify the with the changes you have made in the comment section when accepting.",
						"success",
						true
					)
				);
				toggleRefresh();
				setOpenEditModal(false);
			},
			(err) => {
				dispatch(
					createAlert(
						"Project request update failed",
						"Something went wrong while trying to update the project.",
						"error",
						true
					)
				);
				toggleRefresh();
				setOpenEditModal(false);
			}
		);
	};

	const formItemLayout = {
		labelCol: {
			xs: { span: 24 },
			sm: { span: 8 },
		},
		wrapperCol: {
			xs: { span: 24 },
			sm: { span: 16 },
		},
	};

	return (
		<div style={{ width: "100%" }}>
			{/* Can add the Content thingy to route to be common, but might want to do something about the breadcrumb thingy */}
			<Content
				style={{
					margin: "24px 16px",
					padding: 24,
					background: "#fff",
					minHeight: 280,
				}}
			>
				{/* TODO Change active component to be data display instead of form after submitting change */}

				<Form {...formItemLayout}>
					{/* Project Name */}
					<Form.Item
						label="Project Name"
						style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 400 }}
					>
						{getFieldDecorator("projectName", {
							rules: [
								{
									required: true,
									message: "Please name your project",
								},
							],
							initialValue: request.project.name,
						})(
							<Input
								disabled
								onChange={(event) => setProjectName(event.target.value)}
							/>
						)}
					</Form.Item>
					{/* Project Description */}
					<Form.Item
						label="Project Description"
						style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 450 }}
					>
						<TextArea
							disabled
							defaultValue={request.project.description}
							onChange={(e) => setDesc(e.target.value)}
						/>
					</Form.Item>
					{/* Project Type */}
					<Form.Item
						label="Project Type"
						style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 450 }}
					>
						{getFieldDecorator("projectType", {
							rules: [
								{
									required: true,
								},
							],
							initialValue: request.project.type,
						})(
							<Radio.Group
								disabled
								onChange={(e) => {
									setType(e.target.value);
								}}
							>
								<Radio value="Personal">Personal</Radio>
								<Radio value="Bachelor">Bachelor</Radio>
								<Radio value="Lecture">Lecture</Radio>
							</Radio.Group>
						)}
					</Form.Item>
					{/* Tags Section (handling & displaying) */}
					<Form.Item
						label={
							<span>
								Tags&nbsp;
								<Tooltip title="Tags can be used to identify and search for projects">
									<Icon type="question-circle-o" />
								</Tooltip>
							</span>
						}
						style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 450 }}
					>
						<Input
							value={tagInput}
							onChange={(e) => setTagInput(e.target.value)}
							onPressEnter={handleTagInputConfirm}
						/>
					</Form.Item>
					<div
						style={{
							marginLeft: "auto",
							marginRight: "auto",
							maxWidth: 400,
							width: "100%",
							textAlign: "center",
							marginBottom: 16,
						}}
					>
						<TweenOneGroup
							enter={{
								scale: 0.8,
								opacity: 0,
								type: "from",
								duration: 100,
								onComplete: (e) => {
									e.target.style = "";
								},
							}}
							leave={{ opacity: 0, width: 0, scale: 0, duration: 200 }}
							appear={false}
						>
							{tags.map((tag) => (
								<span key={tag} style={{ display: "inline-block" }}>
									<Tag
										color="blue"
										closable
										onClose={(e) => {
											e.preventDefault();
											removeTag(tag);
										}}
									>
										{tag}
									</Tag>
								</span>
							))}
						</TweenOneGroup>
					</div>
					{/* Author(s) Section */}
					{/* <AuthorInput
						authors={request.project.authors.split(", ")}
						updateAuthors={setAuthors}
					/> */}
					{/* Link(s) Section */}
					{/* <LinkInput links={request.project.links.split(", ")} updateLinks={setLinks} /> */}
					{/* Comment Input */}
					<Form.Item
						label={
							<span>
								Comment&nbsp;
								<Tooltip title="Can be anything, why you want the selected role, why you want to join etc.">
									<Icon type="question-circle-o" />
								</Tooltip>
							</span>
						}
						style={{
							marginLeft: "auto",
							marginRight: "auto",
							maxWidth: 400,
						}}
					>
						<TextArea
							disabled
							defaultValue={request.comment}
							onChange={(event) => setComment(event.target.value)}
						/>
					</Form.Item>
					{/* Button Section */}
					<div style={{ width: "100%", textAlign: "center" }}>
						<Button
							style={{ marginRight: 10 }}
							onClick={(e) => setOpenEditModal(false)}
						>
							Cancel Edit
						</Button>
						<Button type="primary" onClick={(e) => handleSubmit(e)}>
							Submit Changes
						</Button>
					</div>
				</Form>
			</Content>
		</div>
	);
}

export default Form.create()(RequestEditForm);
