import {
	Button,
	Form,
	Icon,
	Input,
	Layout,
	PageHeader,
	Radio,
	Tooltip,
	Typography,
	Spin,
} from "antd";
import React from "react";
import { useDispatch, useSelector } from "react-redux";
import { Redirect } from "react-router-dom";
import { createAlert } from "../../../store/actions/generalActions";
import { axiosRequest } from "../../../util/axiosRequest";

function NewServer(props) {
	// State
	const [title, setTitle] = React.useState("");
	const [desc, setDesc] = React.useState("");
	const [ip, setIp] = React.useState("");
	const [username, setUsername] = React.useState("");
	const [password, setPassword] = React.useState("");
	const [sshKey, setSSHKey] = React.useState("");
	const [radioChecked, setRadioChecked] = React.useState(false);
	// Conditional rendering
	const [sshEnabled, setSSHEnabled] = React.useState(false);
	const [redirect, setRedirect] = React.useState(false);
	const [loading, setLoading] = React.useState(false);

	// validation
	const [validName, setValidName] = React.useState(false);

	// Get antd sub components
	const { Paragraph } = Typography;
	const { Content } = Layout;
	const { TextArea } = Input;

	const { getFieldDecorator } = props.form;

	const isVerified = useSelector((state) => state.auth.isVerified);
	const dispatch = useDispatch();

	const formItemLayout = {
		labelCol: {
			xs: { span: 24 },
			sm: { span: 8 },
		},
		wrapperCol: {
			xs: { span: 24 },
			sm: { span: 16 },
		},
	};

	const routes = [
		{
			path: "profile/dashboard",
			breadcrumbName: "Dashboard",
		},
		{
			path: "profile/servers",
			breadcrumbName: "Your Servers",
		},
		{
			path: "profile/add-server",
			breadcrumbName: "Add Server",
		},
	];

	const handleSubmit = (event) => {
		event.preventDefault();
		setLoading(true);
		if (isVerified) {
			const url =
				process.env.REACT_APP_APPLICATION_URL +
				process.env.REACT_APP_SERVER_MANAGER +
				"/server/new-server";

			axiosRequest(
				"post",
				url,
				{},
				{
					title: title,
					desc: desc,
					ip: ip,
					username: username,
					password: password,
					ssh: sshKey,
				},
				() => {
					//console.log("OK");
					dispatch(
						createAlert(
							"Server Added",
							"Server has been successfully added. You can now add projects you wish to deploy to the server",
							"success",
							true
						)
					);
					setLoading(false);
					setRedirect(true);
				},
				() => {
					dispatch(
						createAlert(
							"Failed to add server",
							"You can't name the server the same as another server. Also double check that the IP, username and password are correct.",
							"error",
							true
						)
					);
					setLoading(false);
				}
			);
		} else {
			dispatch(
				createAlert(
					"Adding Server failed",
					"You can't add a server without verifying your account. Please check your email inbox for a verification email, and follow the instructions.",
					"warning",
					true
				)
			);
		}
	};

	const validateName = (rule, value, callback) => {
		if (value.trim().length <= 0) {
			setValidName(false);
			callback("Please name your project");
		} else if (/^[a-zA-Z0-9-]+$/.test(value)) {
			setValidName(true);
			return callback();
		} else {
			setValidName(false);
			callback(
				"Only letters, numbers and dashes are allowed. Spaces and special characters not allowed"
			);
		}
	};

	return (
		<div style={{ width: "100%" }}>
			<PageHeader
				style={{
					border: "1px solid rgb(235, 237, 240)",
					backgroundColor: "#FFFFFF",
				}}
				title="Add Server"
				breadcrumb={{ routes }}
			>
				<Paragraph>Fill in the form below to add a new server</Paragraph>
			</PageHeader>
			{/* Can add the Content thingy to route to be common, but might want to do something about the breadcrumb thingy */}
			<Content
				style={{
					margin: "24px 16px",
					padding: 24,
					background: "#fff",
					minHeight: 280,
				}}
			>
				{redirect ? <Redirect to="/profile/servers" /> : <div />}
				<Form {...formItemLayout}>
					<Form.Item
						label="Server Title"
						style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 400 }}
					>
						{getFieldDecorator("name", {
							rules: [
								{
									required: true,
									validator: validateName,
								},
							],
						})(<Input onChange={(e) => setTitle(e.target.value)} />)}
					</Form.Item>
					<Form.Item
						label="Server Description"
						style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 400 }}
					>
						<TextArea onChange={(e) => setDesc(e.target.value)} />
					</Form.Item>
					<Form.Item
						label={
							<span>
								IP Address&nbsp;
								<Tooltip title="Servers' IP address">
									<Icon type="question-circle-o" />
								</Tooltip>
							</span>
						}
						style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 400 }}
					>
						{getFieldDecorator("ip", {
							rules: [
								{
									required: true,
									message: "Please specify the servers' ip address",
								},
							],
						})(<Input onChange={(e) => setIp(e.target.value)} />)}
					</Form.Item>
					<Form.Item
						label={
							<span>
								Username&nbsp;
								<Tooltip title="Username for accessing the server">
									<Icon type="question-circle-o" />
								</Tooltip>
							</span>
						}
						style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 400 }}
					>
						{getFieldDecorator("username", {
							rules: [
								{
									required: true,
									message: "Please specify the username",
								},
							],
						})(<Input onChange={(e) => setUsername(e.target.value)} />)}
					</Form.Item>
					<p
						style={{
							color: "red",
							marginLeft: "auto",
							marginRight: "auto",
							maxWidth: 350,
							textAlign: "center",
							marginTop: -10,
							MarginBottom: -10,
						}}
					>
						NB! Password is stored in clear text for the time being
					</p>
					<Form.Item
						label={
							<span>
								Password&nbsp;
								<Tooltip title="Password for accessing the server">
									<Icon type="question-circle-o" />
								</Tooltip>
							</span>
						}
						style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 400 }}
					>
						{getFieldDecorator("password", {
							rules: [
								{
									required: true,
									message: "Please input your password!",
								},
							],
						})(<Input.Password onChange={(e) => setPassword(e.target.value)} />)}
					</Form.Item>
					{/* // TODO Uncomment when SSH key feature is implemented */}
					{/* <Form.Item
						label="SSH?"
						style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 400 }}
					>
						<Radio.Group>
							<Radio
								value="yes"
								onChange={() => {
									setRadioChecked(true);
									setSSHEnabled(true);
								}}
							>
								Yes
							</Radio>
							<Radio
								c
								value="no"
								onChange={() => {
									setRadioChecked(true);
									setSSHEnabled(false);
								}}
							>
								no
							</Radio>
						</Radio.Group>
					</Form.Item> */}
					{sshEnabled ? (
						<Form.Item
							label="SSH Key"
							style={{ marginLeft: "auto", marginRight: "auto", maxWidth: 400 }}
						>
							<TextArea onChange={(e) => setSSHKey(e.target.value)} />
						</Form.Item>
					) : (
						<div />
					)}
					<div
						style={{ width: "100%", textAlign: "center" }}
						onClick={(e) => handleSubmit(e)}
					>
						{loading ? (
							<div>
								<Spin
									style={{
										width: "100%",
										marginLeft: "auto",
										marginRight: "auto",
										marginBottom: 10,
									}}
									indicator={
										<Icon type="loading" style={{ fontSize: 36 }} spin />
									}
								/>
								<p>This might take a while. Please be patient...</p>
							</div>
						) : (
							<Button
								disabled={
									!validName ||
									ip.length <= 0 ||
									username.length <= 0 ||
									password.length <= 0
									// || !radioChecked
								}
								type="primary"
							>
								Add Server
							</Button>
						)}
					</div>
				</Form>
			</Content>
		</div>
	);
}

export default Form.create()(NewServer);
