import {
	Avatar,
	Button,
	Col,
	Divider,
	Input,
	Layout,
	PageHeader,
	Pagination,
	Row,
	Tabs,
	Tag,
	Typography,
} from "antd";
import axios from "axios";
import React, { useEffect } from "react";
import { useSelector } from "react-redux";
import { Link } from "react-router-dom";
import NTNU from "../../assets/image/ntnu.png";
import Navbar from "../../components/Navbar/Navbar";
// Import custom hooks
import useDebounce from "./../../hooks/useDebounce";

function ProfileOrganizationOverview(props) {
	// State
	const [projects, setProjects] = React.useState([]);
	const [search, setSearch] = React.useState("");
	const [organization, setOrganization] = React.useState({});

	// Pagination state for projects
	const [minNumbProjects, setMinNumbProjects] = React.useState(0);
	const [maxNumbProjects, setMaxNumbProjects] = React.useState(10);

	// Redux state
	const isAuthenticated = useSelector((state) => state.auth.token !== null);
	const isVerified = useSelector((state) => state.auth.isVerified);

	// Import sub components from antd
	const { Content } = Layout;
	const { Title, Text, Paragraph } = Typography;
	const { TabPane } = Tabs;
	const { Search } = Input;

	const debouncedSearchTerm = useDebounce(search, 500);

	useEffect(() => {
		const { pathname } = props.location;
		let arr = pathname.split("/");

		const projectsUrl =
			process.env.REACT_APP_APPLICATION_URL +
			process.env.REACT_APP_GENERAL_API +
			"/organization/" +
			arr[2];

		axios.get(projectsUrl).then((response) => {
			setOrganization(response.data);
		});
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, []);

	const handleProjectsPaginationChange = (value) => {
		if (value <= 1) {
			setMinNumbProjects(0);
			setMaxNumbProjects(10);
		} else {
			setMinNumbProjects((value - 1) * 10);
			setMaxNumbProjects(value * 10);
		}
	};

	// Inspired from https://dev.to/gabe_ragland/debouncing-with-react-hooks-jci
	useEffect(
		() => {
			// Make sure we have a value (user has entered something in input)
			if (debouncedSearchTerm) {
				const projectsUrl =
					process.env.REACT_APP_APPLICATION_URL +
					process.env.REACT_APP_GENERAL_API +
					"/organization/" +
					organization.name +
					"/projects/search?q=" +
					search;

				axios.get(projectsUrl).then((response) => {
					setProjects(response.data);
				});
			} else {
				const { pathname } = props.location;
				let arr = pathname.split("/");

				const projectsUrl =
					process.env.REACT_APP_APPLICATION_URL +
					process.env.REACT_APP_GENERAL_API +
					"/organization/" +
					arr[2] +
					"/projects";

				axios.get(projectsUrl).then((response) => {
					setProjects(response.data);
				});
			}
		},
		// This is the useEffect input array
		// Our useEffect function will only execute if this value changes ...
		// ... and thanks to our hook it will only change if the original ...
		// value (searchTerm) hasn't changed for more than 500ms.
		// eslint-disable-next-line react-hooks/exhaustive-deps
		[debouncedSearchTerm]
	);

	return (
		<React.Fragment>
			<Layout>
				{/* Navbar */}
				<Navbar />
				{/* Content */}
				<Content style={{ backgroundColor: "white" }}>
					<Row style={{ marginTop: 20 }}>
						<Col span={14} offset={5}>
							<Row style={{ maxWidth: 1100 }} gutter={[24, 0]}>
								{/* Profile details */}
								<Col xs={24} md={7}>
									<div style={{ textAlign: "center" }}>
										<img
											style={{
												border: "1px solid black",
												padding: 5,
												backgroundColor: "white",
											}}
											className="identicon"
											src={NTNU}
											alt="NTNU logo"
										/>
										<Title level={4}>{organization.name}</Title>
									</div>
								</Col>
								{/* Profile Content */}
								<Col xs={24} md={17}>
									<PageHeader
										style={{
											border: "1px solid rgb(235, 237, 240)",
											backgroundColor: "#FFFFFF",
										}}
										title={organization.name}
										onBack={() => window.history.back()}
										extra={[
											isAuthenticated && isVerified ? (
												<Button key="application">
													<Link
														to={
															"/organization/" +
															organization.name +
															"/membership"
														}
													>
														Request Membership
													</Link>
												</Button>
											) : (
												<div />
											),
										]}
									>
										<Paragraph ellipsis={{ rows: 3, expandable: true }}>
											{organization.description}
										</Paragraph>
									</PageHeader>
									<Tabs defaultActiveKey="1">
										<TabPane tab="Projects" key="1">
											<Search
												placeholder="input search text"
												onChange={(e) => setSearch(e.target.value)}
											/>
											<Divider />
											{projects.length <= 0 ? (
												<p>No projects available</p>
											) : (
												<Row style={{ marginTop: 10 }} gutter={[0, 24]}>
													{projects
														.slice(minNumbProjects, maxNumbProjects)
														.map((project) => (
															<Col
																style={{ padding: 0 }}
																xs={24}
																key={project.id}
															>
																<div key={project.id}>
																	<Col style={{ padding: 0 }}>
																		<Avatar
																			shape="square"
																			size={64}
																			style={{
																				backgroundColor:
																					"#ff99f7",
																				float: "left",
																				verticalAlign:
																					"middle",
																				marginBottom: 24,
																				marginRight: 12,
																				cursor: "pointer",
																			}}
																		>
																			{project.name
																				.charAt(0)
																				.toUpperCase()}
																		</Avatar>
																		<div>
																			<Link to={"#"}>
																				<Text
																					strong
																					style={{
																						cursor:
																							"pointer",
																					}}
																				>
																					{project.name}
																				</Text>
																			</Link>
																			<Paragraph ellipsis>
																				{
																					project.description
																				}
																			</Paragraph>
																			<div>
																				<div
																					style={{
																						float:
																							"left",
																					}}
																				>
																					{project.tags
																						.length >
																					1 ? (
																						project.tags
																							.split(
																								",",
																								2
																							)
																							.map(
																								(
																									tag
																								) => (
																									<span
																										key={
																											tag
																										}
																										style={{
																											display:
																												"inline-block",
																										}}
																									>
																										<Tag color="blue">
																											{
																												tag
																											}
																										</Tag>
																									</span>
																								)
																							)
																					) : (
																						<div />
																					)}
																				</div>
																			</div>
																		</div>
																	</Col>
																	<Divider />
																</div>
															</Col>
														))}
													{projects.length > 10 ? (
														<Pagination
															style={{
																marginBottom: 20,
																textAlign: "center",
															}}
															defaultCurrent={1}
															defaultPageSize={10}
															onChange={
																handleProjectsPaginationChange
															}
															total={projects.length}
														/>
													) : (
														<div />
													)}
												</Row>
											)}
										</TabPane>
									</Tabs>
								</Col>
							</Row>
						</Col>
					</Row>
				</Content>
			</Layout>
		</React.Fragment>
	);
}

export default ProfileOrganizationOverview;
