package no.autopacker.authenticationserver.security;

import com.auth0.jwt.JWT;
import no.autopacker.authenticationserver.entity.User;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Date;
import java.util.stream.Collectors;

import static com.auth0.jwt.algorithms.Algorithm.HMAC512;

/**
 * This filter is used for authentication. Currently it checks for username and password parameters
 * from the URL and calls AuthenticationManager to verify them
 */
public class JwtAuthenticationFilter extends UsernamePasswordAuthenticationFilter {

  private final Logger authLogger = LoggerFactory.getLogger(JwtAuthenticationFilter.class);

  private final AuthenticationManager authenticationManager;

  public JwtAuthenticationFilter(AuthenticationManager authenticationManager) {
    this.authenticationManager = authenticationManager;

    // TODO Remove this when finished implementing the REST Controller
    setFilterProcessesUrl("/auth/login");
  }

  /**
   * Performs an actual authentication
   *
   * @param request interface which exposes getInputStream() method to read body
   * @param response interface to provide HTTP-specific functionality in sending a response
   */
  @Override
  public Authentication attemptAuthentication(
      HttpServletRequest request, HttpServletResponse response) {
    JSONObject credentials = null;
    try {
      credentials =
          new JSONObject(
              request.getReader().lines().collect(Collectors.joining(System.lineSeparator())));

    } catch (IOException e) {
      authLogger.error(e.getMessage());
    }
    var username = "";
    var password = "";
    if (credentials != null) {
      username = credentials.getString("username");
      password = credentials.getString("password");
    }
    return authenticationManager.authenticate(
        new UsernamePasswordAuthenticationToken(username, password));
  }

  /**
   * On successful authentication
   *
   * @param request interface which exposes getInputStream() method to read body
   * @param response interface to provide HTTP-specific functionality in sending a response
   * @param authResult the object returned from the attemptAuthentication method
   */
  @Override
  protected void successfulAuthentication(
      HttpServletRequest request,
      HttpServletResponse response,
      FilterChain filterChain,
      Authentication authResult) {
    var user = ((User) authResult.getPrincipal());

    // Create JWT token
    var token =
        JWT.create()
            .withSubject(user.getUsername())
            .withClaim("role", user.getRole().getName())
            .withClaim("isVerified", user.isVerified())
            .withIssuedAt(new Date(System.currentTimeMillis()))
            .withExpiresAt(new Date(System.currentTimeMillis() + 864000000))
            .sign(HMAC512(SecurityConstants.JWT_SECRET.getBytes()));

    // Add token in response
    response.addHeader(SecurityConstants.TOKEN_HEADER, SecurityConstants.TOKEN_PREFIX + token);
  }
}
