package no.autopacker.authenticationserver.security;

import com.auth0.jwt.JWT;
import no.autopacker.authenticationserver.entity.User;
import no.autopacker.authenticationserver.repository.UserRepository;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.www.BasicAuthenticationFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

import static com.auth0.jwt.algorithms.Algorithm.HMAC512;

/**
 * This filter is used for authorization. This filter is heavily inspired from:
 * https://github.com/dangeabunea/RomanianCoderExamples/tree/master/SpringBootSecurity
 */
public class JwtAuthorizationFilter extends BasicAuthenticationFilter {

    private UserRepository userRepository;

    public JwtAuthorizationFilter(
            AuthenticationManager authenticationManager, UserRepository userRepository) {
        super(authenticationManager);
        this.userRepository = userRepository;
    }

    @Override
    protected void doFilterInternal(
            HttpServletRequest request, HttpServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        // Read the Authorization header, where the JWT token should be
        String header = request.getHeader(SecurityConstants.TOKEN_HEADER);

        if (header != null) {
          // If header does not contain BEARER or is null delegate to Spring impl and exit
          if (header == null || !header.startsWith(SecurityConstants.TOKEN_PREFIX)) {
            chain.doFilter(request, response);
            return;
          }

          // If header is present, try grab user principal from database and perform authorization
          Authentication authentication = getUsernamePasswordAuthentication(header);
          SecurityContextHolder.getContext().setAuthentication(authentication);
        }
        // Continue filter execution
        chain.doFilter(request, response);
    }

    private Authentication getUsernamePasswordAuthentication(String header) {
        String token = header.split(" ")[1];

        // parse the token and validate it
        String username =
                JWT.require(HMAC512(SecurityConstants.JWT_SECRET.getBytes()))
                        .build()
                        .verify(token)
                        .getSubject();

        // Search in the DB if we find the user by token subject (username)
        // If so, then grab user details and create spring auth token using username, pass,
        // authorities/roles
        if (username != null) {
            User user = userRepository.findByUsername(username);
            return new UsernamePasswordAuthenticationToken(username, null, user.getAuthorities());
        }
        return null;
    }
}
