package no.autopacker.authenticationserver.security;

import no.autopacker.authenticationserver.repository.UserRepository;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpMethod;
import org.springframework.security.authentication.dao.DaoAuthenticationProvider;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;

/** In this class we set up basic a security configuration for our application */
@Configuration
@EnableWebSecurity
public class SecurityConfiguration extends WebSecurityConfigurerAdapter {

  private UserPrincipalDetailsService userPrincipalDetailsService;
  private UserRepository userRepository;

  public SecurityConfiguration(
      UserPrincipalDetailsService userPrincipalDetailsService, UserRepository userRepository) {
    this.userPrincipalDetailsService = userPrincipalDetailsService;
    this.userRepository = userRepository;
  }

  /**
   * Uses the returned AuthenticationProvider to try and obtain an AuthenticationManager
   *
   * @param auth SecurityBuilder used to create an AuthenticationManager.
   */
  @Override
  protected void configure(AuthenticationManagerBuilder auth) throws Exception {
    auth.authenticationProvider(authenticationProvider());
  }

  /**
   * Authorizes requests to specific endpoints
   *
   * @param http allows configuring web based security for specific http requests
   */
  @Override
  protected void configure(HttpSecurity http) throws Exception {
    http
        // When using a JWT we dont need csrf and state in session
        .csrf()
        .disable()
        .sessionManagement()
        .sessionCreationPolicy(SessionCreationPolicy.STATELESS)
        .and()
        // Add authentication & authorization filter
        .addFilter(new JwtAuthenticationFilter(authenticationManager()))
        .addFilter(new JwtAuthorizationFilter(authenticationManager(), this.userRepository))
        // Configure access to secured endpoints
        .authorizeRequests()
        .antMatchers(HttpMethod.POST, "/auth/login").permitAll()
        .antMatchers("/auth/changePassword").authenticated()
        .anyRequest().permitAll();
  }

  /**
   * Returns an AuthenticationProvider implementation that retrieves user details from an
   * UserDetailsService
   *
   * @return an AuthenticationProvider implementation that retrieves user details * from an
   *     UserDetailsService
   */
  DaoAuthenticationProvider authenticationProvider() {
    DaoAuthenticationProvider daoAuthenticationProvider = new DaoAuthenticationProvider();
    daoAuthenticationProvider.setPasswordEncoder(passwordEncoder());
    daoAuthenticationProvider.setUserDetailsService(this.userPrincipalDetailsService);
    return daoAuthenticationProvider;
  }

  /**
   * Returns a password encoder (in this case BCrypt)
   *
   * @return a password encoder
   */
  @Bean
  public PasswordEncoder passwordEncoder() {
    return new BCryptPasswordEncoder();
  }
}
